# Copyright (c) 2023, the Compose project contributors.
# SPDX-License-Identifier: GPL-3.0-or-later

"""Docker Compose YAML file generator.

This package provides utilities for programmatically creating and managing Docker Compose
configuration files through Python classes, simplifying the process of defining
and managing Docker Compose configurations.
"""

from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING, Any, Self

import yaml

if TYPE_CHECKING:
    from types import TracebackType


class DockerComposeManager:
    """A class to create and modify Docker Compose YAML files programmatically.

    Supports context manager usage for auto-saving.
    """

    def __init__(self, path: str, version: str = "3.8") -> None:
        """Initialize the manager with a YAML file path. Loads existing file or creates a new one."""
        self.path: str = path
        self.version: str = version
        self._data: dict[str, Any] = {}
        self._dirty: bool = False
        self._load()

    def _load(self) -> None:
        if Path(self.path).exists():
            with Path(self.path).open("r", encoding="utf-8") as f:
                self._data = yaml.safe_load(f) or {}
        if not self._data:
            self._data = {"version": self.version, "services": {}}
        if "services" not in self._data:
            self._data["services"] = {}
        if not isinstance(self._data["services"], dict):
            self._data["services"] = {}

    def create_service(
        self,
        name: str,
        image: str,
        ports: list[str] | None = None,
        environment: dict[str, str] | None = None,
        **kwargs: object,
    ) -> DockerComposeManager:
        """Create a new service in the compose file.

        Returns:
            DockerComposeManager: self (for chaining)

        """
        services = self._data["services"]
        service: dict[str, Any] = {"image": image}
        if ports is not None:
            service["ports"] = ports
        if environment is not None:
            service["environment"] = environment
        service.update(kwargs)
        services[name] = service
        self._dirty = True
        return self

    def modify_service(self, name: str, **kwargs: object) -> DockerComposeManager:
        """Modify an existing service. Raises KeyError if not found.

        Args:
            name (str): Name of the service to modify.
            **kwargs: Key-value pairs to update in the service configuration.

        Raises:
            KeyError: If the service with the given name does not exist.

        Returns:
            DockerComposeManager: self (for chaining)

        """
        services: dict[str, dict[str, Any]] = self._data["services"]
        if name not in services:
            msg: str = f"Service '{name}' not found."
            raise KeyError(msg)
        services[name].update(kwargs)
        self._dirty = True
        return self

    def remove_service(self, name: str) -> DockerComposeManager:
        """Remove a service from the compose file.

        Returns:
            DockerComposeManager: self (for chaining)

        """
        services: dict[str, dict[str, Any]] = self._data["services"]
        if name in services:
            del services[name]
            self._dirty = True
        return self

    def save(self) -> None:
        """Save the current state to the YAML file."""
        with Path(self.path).open("w", encoding="utf-8") as f:
            yaml.dump(self._data, f, sort_keys=False, indent=2)
        self._dirty = False

    def __enter__(self) -> Self:
        """Enter the context manager and return self.

        Returns:
            DockerComposeManager: The instance itself for context management.

        """
        return self

    def __exit__(
        self,
        exc_type: type[BaseException] | None,
        exc_val: BaseException | None,
        exc_tb: TracebackType | None,
    ) -> None:
        """Exit the context manager and save the file if changes were made.

        Args:
            exc_type: The exception type if an exception was raised, None otherwise.
            exc_val: The exception value if an exception was raised, None otherwise.
            exc_tb: The traceback if an exception was raised, None otherwise.

        """
        if self._dirty:
            self.save()
